(load "fold-layout.scm")

(define-macro (let-params PARAMS KEYS . BODY)
  (define (lookup params key)
    (if (null? params)
        #f
        (cond ((assq key (car params)) => cadr)
              (else (lookup (cdr params) key)))))
  (let ((params (gensym)))
    `(let ((,params ,PARAMS))
       (let ,(map
              (lambda (k) `(,k (or (,lookup ,params ',k)
                                   (error "unbound param" ',k))))
              KEYS)
         ,@BODY))))

(define (make-layout x y)
  (cons x y))

(define (layout-x layout)
  (car layout))

(define (layout-y layout)
  (cdr layout))

(define (cartouche-pre-layout tree params layout)
  (let ((x (layout-x layout))
        (y (layout-y layout)))
    (let-params params (margin-left margin-top)
      (make-layout (+ x margin-left) (+ y margin-top))))) 

(define (make-text-x params layout)
  (layout-x layout))

(define (make-text-y params layout)
  (let-params params (text-height)
              (+ text-height (layout-y layout))))

(define (layout-advance-text-line params layout)
  (let-params params (text-height line-spacing)
    (make-layout (layout-x layout)
                 (+ (* text-height line-spacing)
                    (layout-y layout)))))

(define (text-handler text params layout) 
  (values
   (layout-advance-text-line params layout)
   `(tspan
     (@ (x ,(number->string (make-text-x params layout)))
        (y ,(number->string (make-text-y params layout))))     
     ,text)))

(define (p-post tag params old-layout layout kids)
  (values
   layout
   (let-params params (text-height font-family)
     `(text
       (@ (xml:space "preserve")
          (font-size ,(number->string text-height))
          (font-family ,font-family)
          (x ,(number->string (make-text-x params old-layout)))
          (y ,(number->string (make-text-y params old-layout))))
       ,@kids))))

(define (cartouche-post tag params old-layout layout kids)
  (let ((oldx (layout-x old-layout))
        (oldy (layout-y old-layout))
        (newy (layout-y layout)))
    (let-params params (margin-bottom stroke-width line-color
                        page-width)
      (values
       (make-layout oldx (+ newy margin-bottom))
       `(g (rect
            (@ (fill "none") (stroke ,line-color)
               (stroke-width ,(number->string stroke-width))
               (width ,(number->string (- page-width (* 2 oldx))))
               (height ,(number->string
                         (+ (- newy oldy) margin-bottom)))
               (x ,(number->string oldx))
               (y ,(number->string oldy))
               (ry "20"))) ; rounded corners
           ,@kids))))) 

(define (page-post tag params old-layout layout kids)
  (values
   old-layout
   (let-params params (page-width)
     `(svg
       (@ (xmlns "http://www.w3.org/2000/svg")
          (width ,(number->string page-width))
          (height ,(number->string (layout-y layout)))
          (version "1.0"))
       ,@kids))))

(define *cartouche-stylesheet*
  `((page (post . ,page-post))
    (para
     (post . ,p-post))
    (cartouche
     (pre-layout . ,cartouche-pre-layout)
     (post . ,cartouche-post))
    (*text* . ,text-handler)))

(define *default-params*
  '((margin-left 32) (margin-right 32)
    (margin-top 32) (margin-bottom 32)
    (line-spacing 1.0)
    (font-family "Georgia")
    (stroke-width 4)
    (line-color "blue")
    (text-height 64)
    (page-width 660)))

(define (cartouche->svg doc)
  (fold-layout doc *cartouche-stylesheet* *default-params*
               (make-layout 0 0)))

(define cartouche-doc
  '(cartouche (@ (line-color "red")
                 (text-height 56))
              (para "Warning: Smoking Kills")))

(define-macro (assert-equal X Y)
  (let ((x (gensym)) (y (gensym)))
    `(let ((,x ,X) (,y ,Y))
       (or (equal? ,x ,y)
           (begin
             (newline)
             (write ',X)
             (display " = ")
             (write ,x)
             (newline)
             (newline)
             (write ',Y)
             (display " = ")
             (write ,y)
             (newline)
             (newline)
             (error "not equal"))))))

(define-macro (first expr)
  `(call-with-values
       (lambda () ,expr)
     (lambda values (car values))))

(assert-equal (first (cartouche->svg cartouche-doc))
              '(g (rect (@ (fill "none")
                           (stroke "red")
                           (stroke-width "4")
                           (width "660")
                           (height "120.0")
                           (x "0")
                           (y "0")
                           (ry "20")))
                  (text (@ (xml:space "preserve")
                           (font-size "56")
                           (font-family "Georgia")
                           (x "32")
                           (y "88"))
                        (tspan (@ (x "32") (y "88"))
                               "Warning: Smoking Kills"))))

(use-modules (sxml simple))
(define (cartouche->svg-xml doc)
  (sxml->xml (first (cartouche->svg doc))))

(display "all tests pass\n")
